@@ -220,6 +220,7 @@ g_field_info_get_field (GIFieldInfo *field_info,
 	  break;
 	case GI_TYPE_TAG_INT32:
 	case GI_TYPE_TAG_UINT32:
+	case GI_TYPE_TAG_UNICHAR:
 	  value->v_uint32 = G_STRUCT_MEMBER (guint32, mem, offset);
 	  result = TRUE;
 	  break;
@@ -269,9 +270,9 @@ g_field_info_get_field (GIFieldInfo *field_info,
 	      case GI_INFO_TYPE_FLAGS:
 		{
 		  /* FIXME: there's a mismatch here between the value->v_int we use
-		   * here and the glong result returned from g_value_info_get_value().
-		   * But to switch this to glong, we'd have to make g_function_info_invoke()
-		   * translate value->v_long to the proper ABI for an enum function
+		   * here and the gint64 result returned from g_value_info_get_value().
+		   * But to switch this to gint64, we'd have to make g_function_info_invoke()
+		   * translate value->v_int64 to the proper ABI for an enum function
 		   * call parameter, which will usually be int, and then fix up language
 		   * bindings.
 		   */
@@ -399,6 +400,7 @@ g_field_info_set_field (GIFieldInfo     *field_info,
 	  break;
 	case GI_TYPE_TAG_INT32:
 	case GI_TYPE_TAG_UINT32:
+	case GI_TYPE_TAG_UNICHAR:
 	  G_STRUCT_MEMBER (guint32, mem, offset) = value->v_uint32;
 	  result = TRUE;
 	  break;
