@@ -139,6 +139,116 @@ g_typelib_get_dir_entry (GITypelib *typelib,
   return (DirEntry *)&typelib->data[header->directory + (index - 1) * header->entry_blob_size];
 }
 
+static Section *
+get_section_by_id (GITypelib   *typelib,
+		   SectionType  section_type)
+{
+  Header *header = (Header *)typelib->data;
+  Section *section;
+
+  if (header->sections == 0)
+    return NULL;
+
+  for (section = (Section*)&typelib->data[header->sections];
+       section->id != GI_SECTION_END;
+       section++)
+    {
+      if (section->id == section_type)
+	return section;
+    }
+  return NULL;
+}
+
+DirEntry *
+g_typelib_get_dir_entry_by_name (GITypelib *typelib,
+				 const char *name)
+{
+  Section *dirindex;
+  gint i;
+  const char *entry_name;
+  DirEntry *entry;
+
+  dirindex = get_section_by_id (typelib, GI_SECTION_DIRECTORY_INDEX);
+
+  if (dirindex == NULL)
+    {
+      gint n_entries = ((Header *)typelib->data)->n_local_entries;
+      for (i = 1; i <= n_entries; i++)
+	{
+	  entry = g_typelib_get_dir_entry (typelib, i);
+	  entry_name = g_typelib_get_string (typelib, entry->name);
+	  if (strcmp (name, entry_name) == 0)
+	    return entry;
+	}
+      return NULL;
+    }
+  else
+    {
+      guint8 *hash = (guint8*) &typelib->data[dirindex->offset];
+      guint16 index;
+
+      index = _gi_typelib_hash_search (hash, name);
+      entry = g_typelib_get_dir_entry (typelib, index + 1);
+      entry_name = g_typelib_get_string (typelib, entry->name);
+      if (strcmp (name, entry_name) == 0)
+	return entry;
+      return NULL;
+    }
+}
+
+DirEntry *
+g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
+				  gboolean   fastpass,
+				  GType      gtype)
+{
+  Header *header = (Header *)typelib->data;
+  guint n_entries = header->n_local_entries;
+  const char *gtype_name = g_type_name (gtype);
+  DirEntry *entry;
+  guint i;
+  const char *c_prefix;
+
+  /* Inside each typelib, we include the "C prefix" which acts as
+   * a namespace mechanism.  For GtkTreeView, the C prefix is Gtk.
+   * Given the assumption that GTypes for a library also use the
+   * C prefix, we know we can skip examining a typelib if our
+   * target type does not have this typelib's C prefix.
+   *
+   * However, not every class library necessarily conforms to this,
+   * e.g. Clutter has Cogl inside it.  So, we split this into two
+   * passes.  First we try a lookup, skipping things which don't
+   * have the prefix.  If that fails then we try a global lookup,
+   * ignoring the prefix.
+   *
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=564016
+   */
+  c_prefix = g_typelib_get_string (typelib, header->c_prefix);
+  if (fastpass && c_prefix != NULL)
+    {
+      if (g_ascii_strncasecmp (c_prefix, gtype_name, strlen (c_prefix)) != 0)
+	return NULL;
+    }
+
+  for (i = 1; i <= n_entries; i++)
+    {
+      RegisteredTypeBlob *blob;
+      const char *type;
+
+      entry = g_typelib_get_dir_entry (typelib, i);
+      if (!BLOB_IS_REGISTERED_TYPE (entry))
+	continue;
+
+      blob = (RegisteredTypeBlob *)(&typelib->data[entry->offset]);
+      if (!blob->gtype_name)
+	continue;
+
+      type = g_typelib_get_string (typelib, blob->gtype_name);
+      if (strcmp (type, gtype_name) == 0)
+	return entry;
+    }
+  return NULL;
+}
+
 void
 g_typelib_check_sanity (void)
 {
@@ -288,7 +398,7 @@ validate_header_basic (const guint8   *memory,
 
     }
 
-  if (header->major_version != 4 || header->minor_version != 0)
+  if (header->major_version != 4)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -568,16 +678,17 @@ validate_type_blob (GITypelib     *typelib,
   if (simple->flags.reserved == 0 &&
       simple->flags.reserved2 == 0)
     {
-      if (simple->flags.tag >= GI_TYPE_TAG_ARRAY)
+      if (!G_TYPE_TAG_IS_BASIC(simple->flags.tag))
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
-		       "Wrong tag in simple type");
+		       "Invalid non-basic tag %d in simple type", simple->flags.tag);
 	  return FALSE;
 	}
 
       if (simple->flags.tag >= GI_TYPE_TAG_UTF8 &&
+	  simple->flags.tag != GI_TYPE_TAG_UNICHAR &&
 	  !simple->flags.pointer)
 	{
 	  g_set_error (error,
@@ -936,11 +1047,12 @@ validate_constant_blob (GITypelib     *typelib,
     0, /* GSLIST */
     0, /* GHASH */
     0, /* ERROR */
+    4 /* UNICHAR */
   };
   ConstantBlob *blob;
   SimpleTypeBlob *type;
 
-  g_assert (G_N_ELEMENTS (value_size) == GI_TYPE_TAG_ERROR + 1);
+  g_assert (G_N_ELEMENTS (value_size) == GI_TYPE_TAG_N_TYPES);
 
   if (typelib->len < offset + sizeof (ConstantBlob))
     {
@@ -2017,10 +2129,6 @@ _g_typelib_do_dlopen (GITypelib *typelib)
             {
               GString *shlib_full = g_string_new (shlibs[i]);
 
-              /* Prefix with "lib", try both .la and .so */
-              if (!g_str_has_prefix (shlib_full->str, "lib"))
-                g_string_prepend (shlib_full, "lib");
-              g_string_append (shlib_full, ".la");
               module = g_module_open (shlib_full->str, G_MODULE_BIND_LAZY);
               if (module == NULL)
                 {
