@@ -467,7 +467,10 @@ typedef enum
 {
   GI_MARSHALLING_TESTS_FLAGS_VALUE1 = 1 << 0,
   GI_MARSHALLING_TESTS_FLAGS_VALUE2 = 1 << 1,
-  GI_MARSHALLING_TESTS_FLAGS_VALUE3 = 1 << 2
+  GI_MARSHALLING_TESTS_FLAGS_VALUE3 = 1 << 2,
+  GI_MARSHALLING_TESTS_FLAGS_MASK = GI_MARSHALLING_TESTS_FLAGS_VALUE1 |
+                                    GI_MARSHALLING_TESTS_FLAGS_VALUE2,
+  GI_MARSHALLING_TESTS_FLAGS_MASK2 = GI_MARSHALLING_TESTS_FLAGS_MASK
 } GIMarshallingTestsFlags;
 
 GType gi_marshalling_tests_flags_get_type (void) G_GNUC_CONST;
@@ -482,6 +485,27 @@ void gi_marshalling_tests_flags_out (GIMarshallingTestsFlags *flags_);
 
 void gi_marshalling_tests_flags_inout (GIMarshallingTestsFlags *flags_);
 
+/* Flags with no GType */
+
+typedef enum
+{
+  GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE1 = 1 << 0,
+  GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE2 = 1 << 1,
+  GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE3 = 1 << 2,
+  GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_MASK = GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE1 |
+                                            GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_VALUE2,
+  GI_MARSHALLING_TESTS_NO_TYPE_FLAGS_MASK2 = GI_MARSHALLING_TESTS_FLAGS_MASK
+} GIMarshallingTestsNoTypeFlags;
+
+GIMarshallingTestsNoTypeFlags gi_marshalling_tests_no_type_flags_returnv (void);
+
+void gi_marshalling_tests_no_type_flags_in (GIMarshallingTestsNoTypeFlags flags_);
+void gi_marshalling_tests_no_type_flags_in_zero (GIMarshallingTestsNoTypeFlags flags);
+
+void gi_marshalling_tests_no_type_flags_out (GIMarshallingTestsNoTypeFlags *flags_);
+
+void gi_marshalling_tests_no_type_flags_inout (GIMarshallingTestsNoTypeFlags *flags_);
+
 
 /* Structure */
 
@@ -701,9 +725,9 @@ GType gi_marshalling_tests_interface2_get_type (void) G_GNUC_CONST;
 /* Multiple output arguments */
 
 void gi_marshalling_tests_int_out_out (gint *int0, gint *int1);
-void gi_marshalling_tests_int_out_in (gint *int0, gint int1);
+void gi_marshalling_tests_int_three_in_three_out(gint a, gint b, gint c,
+                                                 gint *out0, gint *out1, gint *out2);
 gint gi_marshalling_tests_int_return_out (gint *int_);
-void gi_marshalling_tests_utf8_full_out_out (gchar **utf80, gchar **utf82);
 
 
 /* GError */
@@ -713,6 +737,7 @@ void gi_marshalling_tests_utf8_full_out_out (gchar **utf80, gchar **utf82);
 #define GI_MARSHALLING_TESTS_CONSTANT_GERROR_MESSAGE "gi-marshalling-tests-gerror-message"
 
 void gi_marshalling_tests_gerror(GError **error);
+void gi_marshalling_tests_gerror_array_in(gint *in_ints, GError **error);
 
 /* Overrides */
 
